package com.example.bdlive_flutter_viewer;

import android.app.Activity;
import android.content.Context;
import android.net.Uri;

import androidx.annotation.NonNull;

import com.bytedance.live.common.env.BDLiveConfig;
import com.bytedance.live.common.env.BDLiveEnv;
import com.bytedance.live.common.utils.IntentUtils;
import com.bytedance.live.common.utils.StringUtils;
import com.bytedance.live.sdk.BuildConfig;
import com.bytedance.live.sdk.player.CustomSettings;
import com.bytedance.live.sdk.player.TVULiveRoom;
import com.bytedance.live.sdk.player.TVULiveRoomServer;
import com.bytedance.live.sdk.player.activity.roomview.JoinResultListener;
import com.bytedance.live.sdk.player.activity.roomview.JoinRoomConfig;
import com.bytedance.live.sdk.player.dialog.CommonWebViewDialog;
import com.bytedance.live.sdk.player.listener.LiveRoomStatusListener;
import com.bytedance.live.sdk.player.listener.RedirectPageListener;
import com.example.bdlive_flutter_viewer.vo.BdlActivity;
import com.example.bdlive_flutter_viewer.vo.BdlViewerOpenUrlType;
import com.pandora.ttlicense2.LicenseManager;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import io.flutter.Log;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/**
 * BdliveVodPlayerPlugin
 */
public class BdliveFlutterViewerPlugin implements FlutterPlugin, MethodCallHandler {
    private String TAG = this.getClass().getSimpleName();

    //通知初始化的通道
    public static final String STATIC_CHANNEL = "com.bytedance.flutter_viewer.plugin";
    //发送通道
    public static final String EVENT_CHANNEL_NAME = "com.bytedance.flutter_viewer.listener.event";


    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel channel;
    private FlutterPluginBinding flutterPluginBinding;
    private FlutterEventChannelHelper eventChannelHelper;
    private BdlActivity bdlActivity;
    private JoinRoomConfig joinRoomConfig;
    Map<String, Integer> entranceValueMap = new HashMap<String, Integer>() {{
        put(RedirectPageListener.Entrance.HEADER_ADVERTISEMENT.value, BdlViewerOpenUrlType.pageHeaderAd.value);
        put(RedirectPageListener.Entrance.BANNER_ADVERTISEMENT.value, BdlViewerOpenUrlType.pageAd.value);
        put(RedirectPageListener.Entrance.FLOATING_ADVERTISEMENT.value, BdlViewerOpenUrlType.floatingAd.value);
        put(RedirectPageListener.Entrance.FLOATING_SHOPPING_CARD.value, BdlViewerOpenUrlType.cardAd.value);
    }};

    private final LiveRoomStatusListener roomStatusListener = new LiveRoomStatusListener() {
        @Override
        public void onLiveRoomActivityCreate(Activity activity) {

        }

        @Override
        public void onLiveRoomActivityResume(Activity activity) {
            sendEventToFlutter("onLiveRoomActivityResume", getBdlActivityMap());
        }

        @Override
        public void onLiveRoomActivityPause(Activity activity) {
            sendEventToFlutter("onLiveRoomActivityPause", getBdlActivityMap());
        }

        @Override
        public void onLiveRoomActivityDestroy(Activity activity) {

        }

        @Override
        public void onLiveRoomFullDestroyed(long l) {

        }

        @Override
        public void onFloatViewWillAppear(long activityId) {
            sendEventToFlutter("onFloatViewWillAppear", getBdlActivityMap());
        }

        @Override
        public void onFloatViewWillClose(long activityId) {
            sendEventToFlutter("onFloatViewWillClose", getBdlActivityMap());
        }

        private Map<String, Object> getBdlActivityMap() {
            Map<String, Object> map = new HashMap<>();
            if (bdlActivity != null) {
                map.put("activity", bdlActivity.generateFlutterMap());
            }
            return map;
        }
    };


    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        this.flutterPluginBinding = flutterPluginBinding;
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), STATIC_CHANNEL);
        channel.setMethodCallHandler(this);
        eventChannelHelper = new FlutterEventChannelHelper(flutterPluginBinding.getBinaryMessenger(),
                EVENT_CHANNEL_NAME);

        CustomSettings.Holder.mSettings.setRedirectPageListener((activity, redirectInfo) -> {
            String entrance = redirectInfo.getEntrance();
            Set<String> strings = entranceValueMap.keySet();
            if (strings.contains(entrance)) {
                if (StringUtils.isEmpty(redirectInfo.getUrlRedirectInfo().getUrl())) {
                    //跳转链接为空则不处理
                    return true;
                }
                Map<String, Object> map = new HashMap<>();
                map.put("url", redirectInfo.getUrlRedirectInfo().getUrl());
                map.put("type", entranceValueMap.get(entrance));
                map.put("enableFloating", redirectInfo.getUrlRedirectInfo().isEnableFloating());
                sendEventToFlutter("onOpenUrlCallback", map);
                return true;
            }
            return false;
        });
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        switch (call.method) {
            case "startWithConfiguration":
                String appID = call.argument("appID");
                String appName = call.argument("appName");
                String channel = call.argument("channel");
                String liveLicenseFilePath = call.argument("liveLicenseFilePath");
                String vodLicenseFilePath = call.argument("vodLicenseFilePath");
                String appVersion = call.argument("appVersion");
                boolean isOversea = BuildConfig.IS_OVERSEA;
                if (StringUtils.isEmpty(channel)) {
                    channel = "BdliveFlutterViewerPlugin";
                }
                if (StringUtils.isEmpty(appVersion)) {
                    appVersion = "0.0.1";
                }
                BDLiveConfig bdLiveConfig = new BDLiveConfig.Builder()
                        .setApplicationContext(flutterPluginBinding.getApplicationContext())
                        .setAppId(appID)
                        .setAppName(appName)
                        .setAppChannel(channel)
                        .setAppVersion(appVersion)
                        .setVodLicenseUri(handleLicensePath(vodLicenseFilePath))
                        .setLiveLicenseUri(handleLicensePath(liveLicenseFilePath))
                        .setAppRegion(isOversea ? "singapore" : "china")
                        .setLicenseCallback(new LicenseManager.Callback() {
                            @Override
                            public void onLicenseLoadSuccess(@NonNull String licenseUri, @NonNull String licenseId) {
                                Log.d(TAG, "onLicenseLoadSuccess: " + licenseUri + "-" + licenseId);
                            }

                            @Override
                            public void onLicenseLoadError(@NonNull String licenseUri, @NonNull Exception e, boolean retryAble) {
                                Log.d(TAG, "onLicenseLoadError: " + licenseUri + "-" + e.getMessage());
                            }

                            @Override
                            public void onLicenseLoadRetry(@NonNull String licenseUri) {

                            }

                            @Override
                            public void onLicenseUpdateSuccess(@NonNull String licenseUri, @NonNull String licenseId) {

                            }

                            @Override
                            public void onLicenseUpdateError(@NonNull String licenseUri, @NonNull Exception e, boolean retryAble) {

                            }

                            @Override
                            public void onLicenseUpdateRetry(@NonNull String licenseUri) {

                            }
                        })
                        .build();

                BDLiveEnv.init(bdLiveConfig);
                //License 信息获取。
                result.success("初始化LicenseManager");
                break;
            case "enterLiveRoom":
                Long activityId = call.argument("activityId");
                String token = call.argument("token");
                Integer authMode = call.argument("authMode");
                Boolean isPortrait = call.argument("isPortrait");
                bdlActivity = new BdlActivity(activityId, token, authMode, isPortrait);
                if (!bdlActivity.isValid()) {
                    result.error("-1", "args is not valid: " + bdlActivity, null);
                    return;
                }
                joinRoomConfig = new JoinRoomConfig(bdlActivity.getActivityId(), bdlActivity.getToken(),
                        bdlActivity.getPortrait(),
                        bdlActivity.getAuthMode() == 1 ? TVULiveRoom.TVURoomAuthMode.PUBLIC : TVULiveRoom.TVURoomAuthMode.CUSTOM,
                        new JoinResultListener() {
                            @Override
                            public void onJoinSuccess() {
                                result.success(true);
                            }

                            @Override
                            public void onJoinFailed(int i, String s) {
                                result.success(false);
                            }
                        });
                TVULiveRoom.joinLiveRoom(flutterPluginBinding.getApplicationContext(), joinRoomConfig);
                break;
            case "leaveLiveRoom":
                TVULiveRoom.leaveLiveRoom(flutterPluginBinding.getApplicationContext());
                result.success(true);
                break;
            case "setLiveRoomStatusListener":
                CustomSettings.Holder.mSettings.setLiveRoomStatusListener(roomStatusListener);
                result.success(true);
                break;
            case "manualClickLiveRoomExitBtn":
                TVULiveRoom.manualClickLiveRoomExitBtn(flutterPluginBinding.getApplicationContext());
                result.success(true);
                break;
            case "openUrl":
                String url = call.argument("url");
                Boolean floating = call.argument("enableFloating");
                boolean enableFloating = floating != null && floating;
                if (enableFloating) {
                    TVULiveRoomServer server = TVULiveRoomServer.Holder.getServer();
                    if (server != null) {
                        Context context = server.getContext();
                        if (context instanceof Activity) {
                            CommonWebViewDialog mCommonWebViewDialog = new CommonWebViewDialog(context,
                                    url,
                                    com.bytedance.live.sdk.R.style.TvuLiveBottomDialog);
                            mCommonWebViewDialog.setUrl(url);
                            mCommonWebViewDialog.show();
                        }
                    }
                } else {
                    IntentUtils.attemptViewUri(flutterPluginBinding.getApplicationContext(),
                            Uri.parse(url));
                }
                result.success(true);
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    private void sendEventToFlutter(String eventName, Map<String, Object> map) {
        map.put("methodName", eventName);
        eventChannelHelper.sendToFlutter(map);
    }

    private String handleLicensePath(String path) {
        if (StringUtils.isEmpty(path)) {
            return path;
        }
        if (!path.startsWith("http://") &&
                !path.startsWith("https://")) {
            path = "assets:///flutter_assets/" + path;
        }
        return path;
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
        eventChannelHelper.destroy();
        CustomSettings.Holder.mSettings.setLiveRoomStatusListener(null);
        CustomSettings.Holder.mSettings.setRedirectPageListener(null);
    }
}
