import 'dart:collection';

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:flutter/material.dart';
import '../api/bdlive_flutter_viewer_api.dart';
import 'bdlive_event_channel.dart';
import 'bdlive_room_status_listener_extension.dart';

class BdliveFlutterViewerImpl implements BdliveViewerApi {
  static const MethodChannel _staticChannel =
      MethodChannel('com.bytedance.flutter_viewer.plugin');

  RTCEventChannel? _observerChannel;

  static Future<void> startWithConfiguration(TTSDKConfiguration config) {
    return _staticChannel.invokeMethod(
        'startWithConfiguration', config.toMap());
  }

  @override
  Future<bool> enterLiveRoom(BdlActivityConfig activity) async {
    var result = await _staticChannel.invokeMethod<bool>(
        'enterLiveRoom', activity.toMap());
    return result ?? false;
  }

  @override
  Future<void> leaveLiveRoom() {
    return _staticChannel.invokeMethod('leaveLiveRoom');
  }

  @override
  Future<void> setLiveRoomStatusListener(BdlLiveRoomStatusListener listener) async{
    _observerChannel?.cancel();
    var result = await _staticChannel.invokeMethod("setLiveRoomStatusListener");
    RTCEventChannel channel =
    RTCEventChannel('com.bytedance.flutter_viewer.listener.event');
    channel.listen((String methodName, Map<dynamic, dynamic> dic) {
      debugPrint('dart - listen = $dic  $listener');

      listener.process(methodName, dic);
    });
    _observerChannel = channel;
    return result;
  }

  @override
  Future<void> manualClickLiveRoomExitBtn() {
    return _staticChannel.invokeMethod('manualClickLiveRoomExitBtn');
  }

  @override
  Future<void> openUrl(String url, BdlViewerOpenUrlType type, bool enableFloating) {
    HashMap<String, dynamic> dic = HashMap();
    dic['url'] = url;
    dic['type'] = type.index + 1;
    dic['enableFloating'] = enableFloating;
    return _staticChannel.invokeMethod('openUrl', dic);
  }
}
